#include <Servo.h>
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>

const int RANDOM_PIN = A0;   // Pin wird nicht belegt (wird für Zufallswerte gebraucht)
// A4: Display SDA
// A5: Display SCL
const int MESSUNG_RUHIGE_HAND = A6;       // Pin mit dem der Draht (im Spielfeld) verbunden ist
const int PIEZO_SUMMER_PIN = 11;          // Pin an dem der Piezo ist (analog)
const int FALLENDER_BAUM_MOTOR_PIN = 10;  // Pin des Motors der den Baum (Feld 4) kippen lässt
const int SCHOKOLADEN_AUSGABE_MOTOR_PIN = A3;
const int KAPUTTE_BRUECKE_MOTOR_PIN = 12;

const int STEP = 2;                 // betreibt Wasserspritzen Motor (WSM)
const int DIR = 3;                  // gibt die Drehrichtung des WSM an
const int STEP_2 = 4;               // betreibt Schokoladen Motor (SM)
const int DIR_2 = A7;               // gibt die Drehrichtung des SM an

const int KNOPF_1 = 5;              // Pins an denen die Knöpfe sind (digital)
const int KNOPF_2 = 6;              // Knopf 1 = Enter
const int KNOPF_3 = 7;              // Knopf 2 = geschafft
const int KNOPF_4 = 8;              // Knopf 3 = nicht geschafft
const int KNOPF_5 = 9;              // Knopf 5 = Positionen 

Servo fallenderBaumMotor;           // Initialisierung der Motoren
Servo schokoladenAusgabeMotor;
Servo kaputteBrueckeMotor;

LiquidCrystal_I2C lcd(0x27,20,4);  // set the LCD address to 0x27 for a 16 chars and 2 line display

int spielerPos[] = {0, 0, 0, 0};    // Array mit Spielerpositionen


void setup() {  
  Serial.begin(9600);
  
  pinMode(KNOPF_1, INPUT_PULLUP);          // Pins für Knöpfe als Input initialisieren
  pinMode(KNOPF_2, INPUT_PULLUP);
  pinMode(KNOPF_3, INPUT_PULLUP);
  pinMode(KNOPF_4, INPUT_PULLUP);
  pinMode(KNOPF_5, INPUT_PULLUP);

  pinMode(STEP, OUTPUT);          // initialisiere WSM als Output
  pinMode(DIR, OUTPUT);
  pinMode(STEP_2, OUTPUT);        // initialisiere SM als Output
  pinMode(DIR_2, OUTPUT);

  fallenderBaumMotor.attach(FALLENDER_BAUM_MOTOR_PIN);    // Motoren werden Pin zugewiesen
  schokoladenAusgabeMotor.attach(SCHOKOLADEN_AUSGABE_MOTOR_PIN);
  kaputteBrueckeMotor.attach(KAPUTTE_BRUECKE_MOTOR_PIN);

  lcd.init();                      // initialize the lcd 
  lcd.backlight();

  fallenderBaumMotor.write(120);      // Bringt Motoren in Startpositionen
  kaputteBrueckeMotor.write(110);



  /* Spielstart:
   * Spielankündigung wird ausgegeben, bis Enter gedrückt wird. */
  while(digitalRead(KNOPF_1)==HIGH){
    displayAusgabe("Seid ihr bereit euch in die Klauen des Zufalls zu begeben?");
    displayAusgabe("Drueckt Enter um eure Reise zu beginnen.");
    delay(2000);
  }
  delay(1000);


  /* Spielablauf:
   * Spieler werden nacheinander bewegt und Feldfunktioen aufgerufen (bis einer
   * das Ziel erreicht hat, dann wird nur noch die Runde beendet). Falls zu
   * Beginn eines Zuges Knopf 5 gedrückt wird werden alle Positionen ausgegeben.
   */
 while (spielerPos[0] < 26 && spielerPos[1] < 26
         && spielerPos[2] < 26 && spielerPos[3] < 26){
    for (int i=0; i<4; i++){
      displayAusgabe("Spieler " +String(i+1) + ", druecke Enter.");
      while (digitalRead(KNOPF_1) == HIGH){
        delay(10);
        if (digitalRead(KNOPF_5) == LOW){
          positionenAusgeben();
        }
      }
      laufen(i);
      if(spielerPos[i]!=0){
        displayAusgabe("Spieler " + String(i+1)+ ", Feld " + String(spielerPos[i]) + " (Enter)");
        
         /* Start wenn Knopfdruck (Enter Knopf) */
         while (digitalRead(KNOPF_1) == HIGH){
            delay(10);
         }
        feldfunktion(i);
      }
      delay(500);
    }
  }

  /* Spielende:
   * Sieger wird angekündigt und geehrt. */
  for(int i=0; i<4; i++){
    if(spielerPos[i]>=26){
      displayAusgabe("Herzlichen Glueckwunsch Spieler " +String(i+1)+"!");
    }
  }
  displayAusgabe(F("Der Schatz ist gefunden."));
  tonErfolg();
  displayAusgabe(F("Bereit fuer die naechste Runde?"));
  
  
  wasserSpritzen(0);
  //schokoladenAusgabe();
}

void loop() {

}



/* Spielsteuerung: Textausgabe
 * Die Funktionen displayAusgabe und textScrollen sorgen zusamme dafür,
 * den übergebenen Text über das LCD Display scrollen zu lassen.
 */
void displayAusgabe(String text){
  lcd.begin(16, 2);

  int laenge = text.length();
  
  String text1 = text.substring(0, 37);
  String text2 = text.substring(21, 58);
  String text3 = text.substring(42, 79);
  String text4 = text.substring(63, 100);

  lcd.setCursor(0,0);
  lcd.print(text1);
  delay(1000);

  textScrollen(text1);
  if(laenge > 37){
    textScrollen(text2);
  }
  if(laenge > 58){
    textScrollen(text3);
  }
  if(laenge > 79){
    textScrollen(text3);
  }
}


void textScrollen(String text){
  lcd.clear();
  lcd.print(text);
  for (int positionCounter = 0; positionCounter < text.length()-15; positionCounter++) {
    // scroll one position left:
    lcd.scrollDisplayLeft();
    // wait a bit:
    delay(300);
  }
}


/* Spielsteuerung: Erfolgston
 * Gibt eine Tonfolge aus die Erfolg bei einer Aufgabe/ Quizzfrage
 * oder ähnlichem symbolisiert. */
void tonErfolg(){
  const int C = 261;            // Frequenzen der Töne
  const int E = 329;           
  const int G = 392;           
  const int C2 = 523;

  tone(PIEZO_SUMMER_PIN, C);
  delay(200);
  tone(PIEZO_SUMMER_PIN, E);
  delay(200);
  tone(PIEZO_SUMMER_PIN, G);
  delay(200);
  tone(PIEZO_SUMMER_PIN, C2);
  delay(500);
  noTone(PIEZO_SUMMER_PIN);
}



/* Spielsteuerung: Misserfolgston
 * Gibt eine Tonfolge aus die Misserfolg bei einer Aufgabe/ Quizzfrage
 * oder ähnlichem symbolisiert. */
void tonMisserfolg(){
  const int E = 329;            // Frequenzen der Töne
  const int F = 349;           
  const int Ges = 370;
  const int G = 392;           

  tone(PIEZO_SUMMER_PIN, G);
  delay(500);
  tone(PIEZO_SUMMER_PIN, Ges);
  delay(500);
  tone(PIEZO_SUMMER_PIN, F);
  delay(500);
  for(int i=0; i<50; i++){
    tone(PIEZO_SUMMER_PIN, E);
    delay(10);
    tone(PIEZO_SUMMER_PIN, E-20);
    delay(10);
  }
  noTone(PIEZO_SUMMER_PIN);
}



/* Spielsteuerung: laufen
 * Es wird eine Zahl zwischen eins und drei gewürfelt, anschließend wird der
 * Spieler entsprechend bewegt. Sollte bereits ein anderer Spieler auf dem Feld
 * stehen wird zufällig entschieden ob der Spieler weiter vor oder zurück muss.
 */
void laufen(int spieler){
  /* Würfelt eine zufällige Zahl von 1 bis 5 und gibt sie aus */
  randomSeed(analogRead(RANDOM_PIN));
  int zahl = random(1, 6);
  
  /* Spielerposition wird angepasst */
  spielerPos[spieler] = spielerPos[spieler] + zahl;
  
  displayAusgabe("Du hast eine " + String(zahl) + " gewuerfelt.");

  /* so lange noch wer anderes auf dem Feld steht zufällig vor oder zurück */
  while (((spielerPos[spieler] == spielerPos[0]) && spieler != 0) ||
    ((spielerPos[spieler] == spielerPos[1]) && spieler != 1) ||
    ((spielerPos[spieler] == spielerPos[2]) && spieler != 2) ||
    ((spielerPos[spieler] == spielerPos[3]) && spieler != 3)){
    delay(1500);
    randomSeed(analogRead(RANDOM_PIN));
    if(int(random(1, 3)) == 1){
      spielerPos[spieler] = spielerPos[spieler] + 1;
      displayAusgabe(F("Da steht jemand, gehe einen Schritt weiter."));
    } else{
      spielerPos[spieler] = spielerPos[spieler] - 1;
      displayAusgabe(F("Da steht jemand, gehe einen Schritt zurueck."));
    }
    if(spielerPos[spieler] == 0){
      displayAusgabe(F("Du bist wieder am Start."));
      break;
    }
  }
}



/* Spielsteuerung: Feld zurück
 * Diese Funktion kann aufgerufen werden wenn ein Spieler als Strafe ein Feld
 * zurück gehen muss, steht bereits jemand dort wird der Spieler ein weiteres 
 * Feld zurück gesetzt.
 */
void schrittZurueck(int spieler){
  /* bewegt Spieler einen Schritt zurück */  
  spielerPos[spieler] = spielerPos[spieler] - 1;
  displayAusgabe(F("Gehe einen Schritt zurueck."));

  /* solange Feld nicht frei noch einen Schritt zurück */
  while (((spielerPos[spieler] == spielerPos[0]) && spieler != 0) ||
    ((spielerPos[spieler] == spielerPos[1]) && spieler != 1) ||
    ((spielerPos[spieler] == spielerPos[2]) && spieler != 2) ||
    ((spielerPos[spieler] == spielerPos[3]) && spieler != 3)){
    if(spielerPos[spieler] == 0){
      displayAusgabe(F("Du bist wieder am Start."));
      break;
    }else{
      spielerPos[spieler] = spielerPos[spieler] - 1;
      displayAusgabe(F("Da steht jemand, gehe noch einen Schritt zurueck."));
    }
  }
}



/* Spielsteuerung: Positionen ausgeben
 * Falls im normalen Spielverlauf (außerhalb von Minispielen, Fragen und Aufgaben)
 * Knopf 5 gedrückt wird werden alle Spielerpositionen ausgegeben.
 */
void positionenAusgeben(){
  displayAusgabe(F("Die Spieler stehen auf folgenden Feldern:"));
  displayAusgabe("Spieler 1: Feld " + String(spielerPos[0]));
  displayAusgabe("Spieler 2: Feld " + String(spielerPos[1]));
  displayAusgabe("Spieler 3: Feld " + String(spielerPos[2]));
  displayAusgabe("Spieler 4: Feld " + String(spielerPos[3]));

  delay(500);
  displayAusgabe(F("Zum Wuerfeln Enter druecken."));
  while (digitalRead(KNOPF_1) == HIGH){
        delay(10);
      }
}



/* Spielsteuerung: Feldfunktionen
 * Diese Funktion entscheidet was auf welchem Feld passiert und ruft die
 * entsprechende Funktion auf.
 */
void feldfunktion(int spieler){
  /* Überprüfen auf feste Feldfunktionen */
  if(spielerPos[spieler]==4){
    fallenderBaum(spieler);
  }else if(spielerPos[spieler]==7 || spielerPos[spieler]==9){
    wasserSpritzen(spieler);
  }else if(spielerPos[spieler]==13){
    treppeHoch(spieler);
  }else if(spielerPos[spieler]==19){
    treppeRunter(spieler);
  //}else if(spielerPos[spieler]==16 || spielerPos[spieler]==18){
  //  jellyBeanAusgabe();
  }else if(spielerPos[spieler]==17){
    schokoladenAusgabe();
  }else if(spielerPos[spieler]==22){
    brueckefaellt(spieler);
  }
  /* falls der Spieler auf keinem besonderen Feld ist wird eine Frage oder
     eine Aufgabe gestellt oder eine Minispiel gestartet */
  else{
    /* Zufallszahl wird erstellt */
    randomSeed(analogRead(RANDOM_PIN));
    int zufallszahl = random(1,11);
    /* Unterscheidung zwischen nichts, Frage, Aufgabe oder Minispiel */
    if(zufallszahl==1){
      displayAusgabe(F("Keine Aufgabe fuer dich. Geniesse es!"));
    }else if(zufallszahl==2 || zufallszahl==3 || zufallszahl==4){
      quizzfrage(spieler);
    }else if(zufallszahl==5 || zufallszahl==6 || zufallszahl==7){
      aufgabe(spieler);
    }else if(zufallszahl==8 || zufallszahl==9 || zufallszahl==10){
      /* Zufallszahl um ein Minispiel zu wählen */
      randomSeed(analogRead(RANDOM_PIN));
      int randomzahl = random(1,6);
      /* Fallunterscheidung welches Minispiel startet */
      if(randomzahl==1){
        reaktionsspiel(spieler);
      }else if(randomzahl==2){
        toeneSpielen(spieler);
      }else if(randomzahl==3){
        schereSteinPapier(spieler);
      }else if(randomzahl==4){
        ruhigeHand(spieler);
      }else if(randomzahl==5){
        tannenVonHanoi(spieler);
      }
    }
  }
}



/* Eingebaute Funktion: fallender Baum
 * Baum fällt mit bestimmter Wahrscheinlichkeit um, wenn ein Spieler
 * auf Feld 4 steht.
 */
void fallenderBaum(int spieler){
  /* Erzeugen einer Zufallszahl */
  randomSeed(analogRead(RANDOM_PIN));
  int zahl = random(0,2);

  displayAusgabe(F("Der Wind heult bedrohlich waehrend du den Baum erklimmst."));
  displayAusgabe(F("Druecke Enter wenn du oben bist."));

  delay(500);
  while(digitalRead(KNOPF_1)==HIGH){
    delay(10);
  }

  /* Fallunterscheidung ob Baum kippt */
  if(zahl==1){
    displayAusgabe(F("Glueck gehabt! Du bleibst stehen."));    
  }else{
    spielerPos[spieler] = 0;
    displayAusgabe(F("Du kannst dich nicht halten."));
  
    const int OBERE_GRENZE = 120;   // Grenzen der Motostellung
    const int UNTERE_GRENZE = 50;
    const int PAUSE = 2000;         // Pause bevor der Baum sich wieder aufstellt

    for(int i = OBERE_GRENZE; i >= UNTERE_GRENZE; i--){   // Baum richtet sich langsam auf
      fallenderBaumMotor.write(i);
      delay(20);
    }
    
    delay(PAUSE);
    
    for(int i = UNTERE_GRENZE; i <= OBERE_GRENZE; i++){   // Baum kippt langsam
      fallenderBaumMotor.write(i);
      delay(20);
    }

    displayAusgabe(F("Du bist wieder am Start."));
  }
}



/* Eingebaute Funktion: Wasser spritzen
 * Spieler werden auf Feld 7 und 9 zufällig nass gespritzt.
 */
void wasserSpritzen(int spieler){
  /* Erzeugen einer Zufallszahl */
  randomSeed(analogRead(RANDOM_PIN));
  int zahl = random(0,2);

  /* Fallunterschiedung ob Wasser spritzt */
  if(zahl==1){
    displayAusgabe(F("Du rutscht aus und faellst ins Wasser."));
    
    // Code für die Wasserspritze
    digitalWrite(DIR, HIGH);
    /* zieht am Hebel */
    for(int i = 0; i<700; i++){
      digitalWrite(STEP, HIGH);
      delayMicroseconds(500);
      digitalWrite(STEP, LOW);
      delayMicroseconds(500);
    }
    digitalWrite(DIR, LOW);
    /* löst den Hebel wieder */
    for(int i = 0; i<700 ; i++){
      digitalWrite(STEP, HIGH);
      delayMicroseconds(500);
      digitalWrite(STEP, LOW);
      delayMicroseconds(500);
    }
    
  }else{
    displayAusgabe(F("Glueck gehabt. Du bleibst trocken!"));
  }
}



/* Eingebaute Funktion: Treppe hoch
 * Es wird zufällig entschieden ob ein Spieler auf Feld 13 die Treppe zu Feld 19
 * nehmen kann.
 */
void treppeHoch(int spieler){
  /* Erzeugen einer Zufallszahl */
  randomSeed(analogRead(RANDOM_PIN));
  int zahl = random(0,2);

  /* Fallunterscheidung ob hoch oder nicht */
  if(zahl==1){
    spielerPos[spieler]=19;
    displayAusgabe("Nimm den Tunnel zu Feld 19! (Enter)"); //Tunnel
    while (digitalRead(KNOPF_1) == HIGH){
       delay(10);
    }
  }else{
    displayAusgabe("Bleibe stehen (Enter)");
    while (digitalRead(KNOPF_1) == HIGH){
       delay(10);
    }
  }
}



/* Eingebaute Funktion: Treppe runter
 * Es wird zufällig entschieden ob ein Spieler auf Feld 19 die Treppe zu Feld 13
 * nehmen muss.
 */
void treppeRunter(int spieler){
  /* Erzeugen einer Zufallszahl */
  randomSeed(analogRead(RANDOM_PIN));
  int zahl = random(0,2);

  /* Fallunterscheidung ob runter oder nicht */
  if(zahl==1){
    spielerPos[spieler]=13;
    displayAusgabe(F("Nimm den Tunnel zu Feld 13! (Enter)"));
    while (digitalRead(KNOPF_1) == HIGH){
       delay(10);
    }
  }else{
    displayAusgabe(F("Bleibe stehen (Enter)"));
    while (digitalRead(KNOPF_1) == HIGH){
        delay(10);
    }
  }
}



/* Eingebaute Funktion: Jelly-Bean-Ausgabe 
 * Auf Feld 15 und 17 im Schlaraffenland wird eine Jelly Bean ausgegeben.
 */
void jellyBeanAusgabe(){
  /* Code für jellyBeanAusgabe */
  
  displayAusgabe(F("Guten Appetit... (Enter)"));
  while (digitalRead(KNOPF_1) == HIGH){
            delay(10);
  }
  
}



/* Eingebaute Funktion: Schokoladenausgabe 
 * Auf Feld 16 im Schlaraffenland wird Schokolade ausgegeben.
 */
void schokoladenAusgabe(){
  digitalWrite(DIR_2, HIGH);

  for(int i = 0; i<200; i++){     // eine motor Umdrehung
    digitalWrite(STEP_2, HIGH);
    delay(20);
    digitalWrite(STEP_2, LOW);
    delay(20);
  }

  displayAusgabe(F("Guten Appetit! (Enter)"));
  while (digitalRead(KNOPF_1) == HIGH){
     delay(10);
  }
}



/* Eingebaute Funktion: kaputte Brücke
 * Es wird zufällig entschieden ob die Brücke (Feld 22) kaputt geht, falls
 * jemand drauf steht (Spieler setzt danach eine Runde aus).
 */
void brueckefaellt(int spieler){
  /* Erzeugen einer Zufallszahl */
  randomSeed(analogRead(RANDOM_PIN));
  int zahl = random(0,2); 

  /* Fallunterscheidung ob Brücke kaputt geht */
  if(zahl==1){
    /* Code für die Hängebrückenfalltür */
    const int OBERE_GRENZE = 110;   // Grenzen der Motorstellung
    const int UNTERE_GRENZE = 80;
    const int PAUSE = 2000;         // Pause bevor die Brücke sich wieder ausfährt
    
    kaputteBrueckeMotor.write(UNTERE_GRENZE); // Brücke zieht sich schnell zurück
    
    delay(PAUSE);

    for(int i = UNTERE_GRENZE; i <= OBERE_GRENZE; i++){   // Brücke fährt langsam wieder aus
      kaputteBrueckeMotor.write(i);
      delay(30);
    }
        
    spielerPos[spieler] = 21;
    displayAusgabe(F("Du bist gestuerzt! Klettere zurueck zur 21. (Enter)"));
    while (digitalRead(KNOPF_1) == HIGH){
       delay(10);
    }
  }else{
    displayAusgabe(F("Die Bruecke ist stabil. (Enter)"));
    while (digitalRead(KNOPF_1) == HIGH){
       delay(10);
    }
  }
}



/* Minispiel: Reaktionsspiel
 * Nach zufälliger Zeit wird ein Ton ausgegeben, der Spieler muss innerhalb
 * einer bestimmten Zeit den Knopf drücken. Bei Misserfolg muss der Spieler
 * einen Schritt zurück gehen.
 */
void reaktionsspiel (int spieler){
  /* Deklaration von Variablen */
  int reaktionsZeit = 175;          // Zeit in der reagiert werden muss
  boolean geschafft = false;        // Speichert ob Aufgabe erledigt wurde
  int reaktion;                     // Zeit in der der Spieler reagiert
 
  /* Spiel wird über Display angekündigt */
  displayAusgabe(F("Minispiel: Reaktionsspiel"));
  delay(500);
  displayAusgabe(F("Zum Starten Enter druecken."));
  while (digitalRead(KNOPF_1) == HIGH){
    delay(100);
  }

  /* nach zufälliger Zeit wird Startsignal ausgegeben */
  randomSeed(analogRead(RANDOM_PIN));
  delay(random(2000, 5000));
  tone(PIEZO_SUMMER_PIN, 261, 100);

  /* Schleife, die in einer bestimmten Zeit auf Knopfdruck überprüft */
  for (int zeit = 0; zeit < reaktionsZeit; zeit++){
    if (digitalRead(KNOPF_1) == LOW){
      geschafft = true;
      reaktion = zeit+1;
      break;
    }
    delay(1);
  }

  delay(1000);

  /* Ausgabe des Ergebnisses und Konsequenzen */
  if (geschafft == true){
    displayAusgabe("Herzlichen Glueckwunsch, du Flash. (" +String(reaktion)+ "ms)");
    tonErfolg();
  } else{
    displayAusgabe(F("Das war zu langsam, du Schnecke."));
    tonMisserfolg();
    schrittZurueck(spieler);
  }
}



/* Minispiel: Töne nachspielen
 * Der Roboter gibt 5 zufällige Töne aus, der Spieler muss diese mit den fünf
 * Knöpfen nachspielen. Bei Misserfolg muss der Spieler einen Schritt zurück.
 */
void toeneSpielen (int spieler){
  /* Deklaration von Variablen */
  const int ANZAHL_TOENE = 5;       // Länge der Tonabfolge
  const int DAUER = 500;            // Dauer des Tons in ms
  const int PAUSE = 500;            // Dauer der Pause zwischen Tönen in ms
  const int TON_1 = 261;            // Frequenz vom ersten Ton (C4)
  const int TON_2 = 293;            // Frequenz vom zweiten Ton (D4)
  const int TON_3 = 329;            // Frequenz vom dritten Ton (E4)
  const int TON_4 = 349;            // Frequenz vom vierten Ton (F4)
  const int TON_5 = 392;            // Frequenz vom fünften Ton (G4)
  
  /* Spiel wird über Display angekündigt */
  displayAusgabe(F("Minispiel: Toene nachspielen"));
  delay(500);
  displayAusgabe(F("Zum Starten Enter druecken."));

  /* Start wenn Knopfdruck (Enter Knopf) */
  while (digitalRead(KNOPF_1) == HIGH){
    delay(10);
  }
  
  displayAusgabe(F("So klingt ein C:"));
  tone(PIEZO_SUMMER_PIN, TON_1);
  delay(1000);
  noTone(PIEZO_SUMMER_PIN);
  displayAusgabe(F("Spiele die folgende Melodie nach."));
  delay(1000);

  /* Array von Zufällstönen wird erstellt */
  int roboToene[ANZAHL_TOENE];
  for (int i=0; i < ANZAHL_TOENE; i++){
      randomSeed(analogRead(RANDOM_PIN));
      roboToene[i] = random(5);
      delay(100);
    }

  /* Töne werden über Piezosummer ausgegeben */
  for (int i=0; i < ANZAHL_TOENE; i++){
    if (roboToene[i] == 0){
        tone(PIEZO_SUMMER_PIN, TON_1);
        delay(PAUSE);
        noTone(PIEZO_SUMMER_PIN);
        delay(500);
    }else if (roboToene[i] == 1){
        tone(PIEZO_SUMMER_PIN, TON_2);
        delay(PAUSE);
        noTone(PIEZO_SUMMER_PIN);
        delay(500);
    }else if (roboToene[i] == 2){
        tone(PIEZO_SUMMER_PIN, TON_3);
        delay(PAUSE);
        noTone(PIEZO_SUMMER_PIN);
        delay(500);
    }else if (roboToene[i] == 3){
        tone(PIEZO_SUMMER_PIN, TON_4);
        delay(PAUSE);
        noTone(PIEZO_SUMMER_PIN);
        delay(500);
    }else if (roboToene[i] == 4){
        tone(PIEZO_SUMMER_PIN, TON_5);
        delay(PAUSE);
        noTone(PIEZO_SUMMER_PIN);
        delay(500);
    }
  }
  delay(1000);

  /* Eingabe des Spielers wird erfasst */
  int spielerToene[] = {10, 10, 10, 10, 10};
  int i = 0;
  while (i < ANZAHL_TOENE){
    if (digitalRead(KNOPF_1) == LOW){
      tone(PIEZO_SUMMER_PIN, TON_1, DAUER);
      spielerToene[i] = 0;
      delay(DAUER);
    }else if (digitalRead(KNOPF_2) == LOW){
      tone(PIEZO_SUMMER_PIN, TON_2, DAUER);
      spielerToene[i] = 1;
      delay(DAUER);
    }else if (digitalRead(KNOPF_3) == LOW){
      tone(PIEZO_SUMMER_PIN, TON_3, DAUER);
      spielerToene[i] = 2;
      delay(DAUER);
    }else if (digitalRead(KNOPF_4) == LOW){
      tone(PIEZO_SUMMER_PIN, TON_4, DAUER);
      spielerToene[i] = 3;
      delay(DAUER);
    }else if (digitalRead(KNOPF_5) == LOW){
      tone(PIEZO_SUMMER_PIN, TON_5, DAUER);
      spielerToene[i] = 4;
      delay(DAUER);
    }
    if (spielerToene[i] != 10){     // falls ein Ton erfasst wurde wird i erhöht
      i++; 
    }
  }

  /* Eingabe des Spielers wird überprüft */
  boolean geschafft = true;               // speichert ob Aufgabe erledigt wurde
  for (int i=0; i < ANZAHL_TOENE; i++){   // vergleicht die Eingabe und Ausgabe Arrays
    if (roboToene[i] != spielerToene[i]){
      geschafft = false;
    }
  }

  /* Ergebnis wird bekanntgegeben und Konsequenzen werden wenn nötig gezogen */
  if (geschafft == true){
    displayAusgabe(F("Das war richtig!"));
    tonErfolg();
  }else{
    displayAusgabe(F("Das war leider falsch..."));
    tonMisserfolg();
    schrittZurueck(spieler);
  }
}



/* Minispiel: Sching Schang Schong
 * Spieler muss gegen den Roboter Sching Schang Schong spielen, bei einem
 * Unendschieden wird erneut gespielt bis einer gewinnt. Bei Misserfolg muss
 * der Spieler einen Schritt zurück gehen.
 */
// Spieler spielen Sching Schang Schong
void schereSteinPapier(int spieler){  
  /* Spielanweisung */
  delay(500);
  displayAusgabe(F("Minispiel: Schere, Stein, Papier"));
  displayAusgabe(F("Knopf 1 = Schere"));
  displayAusgabe(F("Knopf 2 = Stein"));
  displayAusgabe(F("Knopf 3 = Papier"));

  /* Warten bis Knopf gedrückt wird */
  while (digitalRead(KNOPF_1) == HIGH && digitalRead(KNOPF_2) == HIGH
         && digitalRead(KNOPF_3) == HIGH){
    delay(10);
  }

  /* Computer wählt */
  randomSeed(analogRead(RANDOM_PIN));
  int randomZahl = random(1,4);     // 1 = Schere, 2= Stein, 3= Papier

  /* Einlesen von Spielerwahl + Konsequenzen */
  /* Computer wählt Schere */
  if(randomZahl==1){
    if(digitalRead(KNOPF_1) == LOW){
      displayAusgabe(F("Unentschieden (Schere vs. Schere), versuche es nochmal."));
      schereSteinPapier(spieler);
    }else if(digitalRead(KNOPF_2) == LOW){
      displayAusgabe(F("Du hast gewonnen (Schere vs. Stein)."));
      tonErfolg();
    }else{
      displayAusgabe(F("Du hast verloren (Schere vs. Papier)."));
      tonMisserfolg();
      schrittZurueck(spieler);
    }
  }
  /* Computer wählt Stein */
  if(randomZahl==2){
    if(digitalRead(KNOPF_2) == LOW){
      displayAusgabe(F("Unentschieden (Stein vs. Stein), versuche es nochmal."));
      schereSteinPapier(spieler);
    }else if(digitalRead(KNOPF_3) == LOW){
      displayAusgabe(F("Du hast gewonnen (Stein vs. Papier)."));
      tonErfolg();
    }else{
      displayAusgabe(F("Du hast verloren (Stein vs. Schere)."));
      tonMisserfolg();
      schrittZurueck(spieler);
    }  
  }
  /* Computer wählt Papier */
  if(randomZahl==3){
    if(digitalRead(KNOPF_3) == LOW){
      displayAusgabe(F("Unentschieden (Papier vs. Papier), versuche es nochmal."));
      schereSteinPapier(spieler);
    }else if(digitalRead(KNOPF_1) == LOW){
      displayAusgabe(F("Du hast gewonnen (Papier vs. Schere)."));
      tonErfolg();
    }else{
      displayAusgabe(F("Du hast verloren (Papier vs. Stein)."));
      tonMisserfolg();
      schrittZurueck(spieler);
    }   
  }
}



/* Minispiel: Ruhige Hand 
 * Der Spieler muss den Hackenstab durch den gesammten Draht auf dem Spielbrett
 * bewegen ohne diesen dabei zu berühren.
 */
void ruhigeHand(int spieler){
  /* Draht auf Spielbrett wird unter Strom gesetzt. */
  pinMode(MESSUNG_RUHIGE_HAND, INPUT_PULLUP);   // hat immer Strom, außer wenn mit gnd verbunden
  
  /* Deklaration von Variablen */
  boolean gewonnen = true;
  
  /* Spiel wird über Display angekündigt */
  displayAusgabe(F("Minispiel: Ruhige Hand"));
  delay(500);
  displayAusgabe(F("Zum Starten und beenden Enter druecken."));
  while(digitalRead(KNOPF_1)==HIGH){
    delay(10);
  }
  displayAusgabe(F("Los gehts! (Enter)"));
   while(digitalRead(KNOPF_1)==HIGH){
    delay(10);
  }

  /* Spielstart: Hakenstock wird mit Ground verbunden
     Prüfen, Pin LOW wird -> Kontakt zwischen Parkour und Haken */
  while(digitalRead(KNOPF_1)==HIGH){
    if(analogRead(MESSUNG_RUHIGE_HAND)==LOW){
      gewonnen = false;
      tone(PIEZO_SUMMER_PIN, 261);
    }else{
      noTone(PIEZO_SUMMER_PIN);
    }    
  }

  /* Strom am Draht wird wieder aus geschaltet. */
  pinMode(MESSUNG_RUHIGE_HAND, INPUT);
  
  /* Ergebnis wird bekanntgegeben und Konsequenzen werden wenn nötig gezogen */
  if (gewonnen == true){
    displayAusgabe(F("Du hast eine ruhige Hand!."));
    tonErfolg();
  }else{
    displayAusgabe(F("Du hast den Draht beruehrt. Loser."));
    tonMisserfolg();
    schrittZurueck(spieler);
  }
}



/* Minispiel: Die Tannen von Hanoi
 * Es wird zufällig entschieden mit wie vielen Scheiben gespielt wird, dann
 * muss der Spieler die Türme von Hanoi spielen und seinen Erfolg oder
 * Misserfolg angeben. */
void tannenVonHanoi(int spieler){
  randomSeed(analogRead(RANDOM_PIN));
  int zahl = random(4, 7);
  displayAusgabe(F("Minispiel: Die Tannen von Hanoi"));
  displayAusgabe("Du musst mit " + String(zahl) + " Ringen spielen.");
  float potenz = zahl-4;
  double zeit = 25*pow(2.00,potenz);
  displayAusgabe("Du hast " + String(zeit) + " Sekunden Zeit.");
  delay(500);
  displayAusgabe(F("Zum Starten Enter druecken."));
  while(digitalRead(KNOPF_1)==HIGH){
    delay(10);
  }
  
  
  delay((zeit-10)*1000);
  for(int i = 0; i < 10; i++){    // letzten 10sek mit Ticken
    noTone(PIEZO_SUMMER_PIN);
    delay(950);
    tone(PIEZO_SUMMER_PIN, 261);
    delay(50);
  }
  delay(950);                      // 1 Sekunde Dauerton bei Zeitende
  noTone(PIEZO_SUMMER_PIN);

/* Ergebnis wird eingelesen und Konsequenzen gezogen */
  displayAusgabe(F("Geschafft? Gruen! Nicht geschafft? Rot!"));
  while (digitalRead(KNOPF_2) == HIGH && digitalRead(KNOPF_3) == HIGH ){
    delay(10);
  }
  if(digitalRead(KNOPF_2)==LOW){
    displayAusgabe(F("Herzlichen Glueckwunsch, du Brain."));
    tonErfolg();
  }else{
    displayAusgabe(F("Das war wohl nichts."));
    tonMisserfolg();
    schrittZurueck(spieler);
  }
}



/* Quizzfragen:
 * Spieler muss eine Frage beantworten, bei falscher Antwort muss er einen
 * Schritt zurück gehen.
 */
void quizzfrage(int spieler){  
  /* Tabelle mit Fragen im flash Speicher */
  const static char fragen_0[] PROGMEM = "Nachname Queen Elizabeth II?";
  const static char fragen_1[] PROGMEM = "Absoluter Nullpunkt in Grad Celsius?";
  const static char fragen_2[] PROGMEM = "F in J. F. Kennedy steht fuer...?";
  const static char fragen_3[] PROGMEM = "Vorfahren der Goetter (Gr. Myth.)?";
  const static char fragen_4[] PROGMEM = "Wer haelt den Himmel (Gr. Myth.)?";
  const static char fragen_5[] PROGMEM = "Buchstabiere den Namen von Thors Hammer:";
  const static char fragen_6[] PROGMEM = "Buchstabiere den Namen von Islands Hauptstadt:";
  const static char fragen_7[] PROGMEM = "Gott hat einen harten linken Haken – Welches Lied?";
  const static char fragen_8[] PROGMEM = "Saenger der Beatles?";
  const static char fragen_9[] PROGMEM = "Elementensymbol Zinn?";
  const static char fragen_10[] PROGMEM = "Elementensymbol Einsteinium?";
  const static char fragen_11[] PROGMEM = "Elementensymbol Stickstoff?";
  const static char fragen_12[] PROGMEM = "Synonym fuer Esskastanie?";
  const static char fragen_13[] PROGMEM = "Was ist ein Nasenfahrrad?";
  const static char fragen_14[] PROGMEM = "Zauberstab auf Franzoesisch?";
  const static char fragen_15[] PROGMEM = "Wort des Jahres 1989?";
  const static char fragen_16[] PROGMEM = "Wie viele Zeilen Code hat dieses Programm? (+/- 100)";
  const static char fragen_17[] PROGMEM = "US-Schuhgroesse 8,5 (Damen) in EU-Groesse?";
  const static char fragen_18[] PROGMEM = "US-Schuhgroesse 8,5 (Herren) in EU-Groesse?";
  const static char fragen_19[] PROGMEM = "Welche Voegel legen niemals Eier?";
  const static char fragen_20[] PROGMEM = "Hauptstadt von Kanada?";
  const static char fragen_21[] PROGMEM = "Hauptstadt von Slowenien?";
  const static char fragen_22[] PROGMEM = "Frueherer Name von Istanbul?";
  const static char fragen_23[] PROGMEM = "Frueherer Name von New York?";
  const static char fragen_24[] PROGMEM = "Gruendungsjahr von Rom?";
  const static char fragen_25[] PROGMEM = "Autor von Emil und die Detektive?";
  const static char fragen_26[] PROGMEM = "Autor von Karlsson vom Dach?";
  const static char fragen_27[] PROGMEM = "Wie heisst Pippi Langstrumpfs Papa?";
  const static char fragen_28[] PROGMEM = "Wie heisst der Affenkoenig aus Mogli?";
  const static char fragen_29[] PROGMEM = "Wie heisst der kleinste Knochen des Menschen?";
  const static char fragen_30[] PROGMEM = "Wie lang ist der Darm eines Erwachsenen in m durchschnittlich? (+/- 1)";
  const static char fragen_31[] PROGMEM = "Sage Pi (5 Nachkommastellen):";
  const static char fragen_32[] PROGMEM = "Wofuer steht DHL?";
  const static char fragen_33[] PROGMEM = "Wofuer steht Haribo?";
  const static char fragen_34[] PROGMEM = "Wie viele Tasten hat ein durchschnittliches Klavier?";
  const static char fragen_35[] PROGMEM = "Die Formel fuer kinetische Energie ist...?";
  const static char fragen_36[] PROGMEM = "Was ist das Gegenteilchen zum Elektron?";
  const static char fragen_37[] PROGMEM = "Wie viele mm wächst unser Haar pro Tag? (+/- 0,1)";
  const static char fragen_38[] PROGMEM = "Name der Achse, um die sich das Fahrradrad dreht?";
  const static char fragen_39[] PROGMEM = "Was symbolisiert der Osterhase?";
  const static char fragen_40[] PROGMEM = "Wer sagte: Gott wuerfelt nicht.?";
  const static char fragen_41[] PROGMEM = "Wie hiess die Frau von Albert Einstein?";
  const static char fragen_42[] PROGMEM = "Wessen weise Worte sind: Traue keiner Statistik, die du nicht selbst gefaelscht hast?";     
  const static char fragen_43[] PROGMEM = "Wer wird als der Sonnenkoenig bezeichnet?";
  const static char fragen_44[] PROGMEM = "Wer war der Nachfolger Karls des Großen?";
  const static char fragen_45[] PROGMEM = "Aus welchen vier Basen ist die DNA aufgebaut? ";
  const static char fragen_46[] PROGMEM = "Welches Land beginnt mit X?";
  const static char fragen_47[] PROGMEM = "Welches ist das kleinste Land der Welt?";
  const static char fragen_48[] PROGMEM = "Wie viele Liter Bier werden pro Kopf in Deutschland jaehrlich verbraucht?";    
  const static char fragen_49[] PROGMEM = "Wie viele Einkerbungen hat ein Golfball?";
  const static char fragen_50[] PROGMEM = "Wie lange dauerte der 30-jaehrige Krieg?";
  const static char fragen_51[] PROGMEM = "Was ist der Name von C6H12O6? Chemische Formel)";
  const static char fragen_52[] PROGMEM = "Was ist das Endprodukt der Atmungskette?";
  const static char fragen_53[] PROGMEM = "Wofuer steht FAME-GAMES?";
  const static char fragen_54[] PROGMEM = "Von wem bekommt Harry Potter seinen ersten Besen geschenkt?";
  const static char fragen_55[] PROGMEM = "Wie fing Harry Potter seinen ersten Schnatz?";
  const static char fragen_56[] PROGMEM = "Wie lautet Voldemorts echter Name?";
  const static char fragen_57[] PROGMEM = "Wie nennt man Nichtmagier in England?";
  const static char fragen_58[] PROGMEM = "Welches ist das kleinste Bundesland in Deutschland?";     
  const static char fragen_59[] PROGMEM = "Was bedeutet carpe diem?";
  const static char fragen_60[] PROGMEM = "Welcher Pilz ist der giftigste der Welt?";
  const static char fragen_61[] PROGMEM = "Welches Vitamin wird mithilfe von Sonnenlicht im Koerper gebildet?";
  const static char fragen_62[] PROGMEM = "Wie heisst die Krankheit die durch Vitamin C-Mangel ausgelöst wird?";
  const static char fragen_63[] PROGMEM = "Welches Chromosom liegt beim Down-Syndrom dreifach vor?";
  const static char fragen_64[] PROGMEM = "Was ist der häufigste Vorname der Welt?";
  const static char fragen_65[] PROGMEM = "Was ist das Jugendwort des Jahres 2021 in Deutschland?";
  const static char fragen_66[] PROGMEM = "Auf welchem Planeten ist der hoechste Berg im Sonnensystem?";
  const static char fragen_67[] PROGMEM = "Wie viel Mal laenger als die Menschen haben die Dinosaurier den Planeten dominiert?";
  const static char fragen_68[] PROGMEM = "Wie schreibt man die Zahl 124 (Basis 10) als Zahl mit Basis fuenf?";
  const static char fragen_69[] PROGMEM = "Wie viele Woerter hat der laengste jemals gedruckte Satz?";
  const static char fragen_70[] PROGMEM = "Was ist eine Lichtmaschine?";
  const static char fragen_71[] PROGMEM = "Wie viel Prozent der Welt ernaehren sich vegetarisch?";
  const static char fragen_72[] PROGMEM = "In welcher Stadt lebt die gelbe Zeichentrickfamilie Simpson?";
  const static char fragen_73[] PROGMEM = "Was bedeutet Tiramisu woertlich?";
  const static char fragen_74[] PROGMEM = "Wie viele Herzen hat ein Oktopus?";
  const static char fragen_75[] PROGMEM = "In welchem Jahr wurde Buddha geboren?";
  const static char fragen_76[] PROGMEM = "Woraus besteht Captain Americas Schild?";
  const static char fragen_77[] PROGMEM = "Wie viele Stunden pro Tag schlaeft ein Koala? (+/- 3)";
  const static char fragen_78[] PROGMEM = "Wie viel laenger ist die Zunge eines Chamaeleons im Vgl. zum Koerper?";
  const static char fragen_79[] PROGMEM = "Was ist das Nationaltier von Schottland?";
  const static char fragen_80[] PROGMEM = "Wie urinieren Haie?";
  const static char fragen_81[] PROGMEM = "Welches Gemüse faerbt durch ihren Verzehr die Milch einer Kuh rosa?";
  const static char fragen_82[] PROGMEM = "Was bedeutet die Einheit mmHg?";
  const static char fragen_83[] PROGMEM = "Wie viele Zaehne hat der Mensch durchschnittlich?";
  const static char fragen_84[] PROGMEM = "Wo wurde das Schach erfunden?";
  const static char fragen_85[] PROGMEM = "Wie viel Prozent der Welt sind Linkshaender? (+/-3)";
  const static char fragen_86[] PROGMEM = "In welchem Jahr wurde das Internet entwickelt?";
  const static char fragen_87[] PROGMEM = "Wofür steht GmbH?";
  const static char fragen_88[] PROGMEM = "Welches Gebirge teilt Europa und Asien?";
  const static char fragen_89[] PROGMEM = "Welches Maedchen hat eine kranke Grossmutter im Wald?";
  const static char fragen_90[] PROGMEM = "Wie heißen die Neffen von Donald Duck?";
  const static char fragen_91[] PROGMEM = "Was bezeichnet man als Entomologie?";
  const static char fragen_92[] PROGMEM = "Wie hoch ist der Fernsehturm?";
  const static char fragen_93[] PROGMEM = "Wie beginnt Artikel 1 des deutschen Grundgesetzes?";
  const static char fragen_94[] PROGMEM = "Wie lautet der Satz des Pythagoras?";
  const static char fragen_95[] PROGMEM = "Wo liegen die Langerhans-Inseln?";
  const static char fragen_96[] PROGMEM = "Was ist Dihydrogenmonoxid?";
  const static char fragen_97[] PROGMEM = "Wer hoert alles und sagt nichts?";
  const static char fragen_98[] PROGMEM = "Was schmeckt besser als es riecht?"; 
  const static char fragen_99[] PROGMEM = "Wessen weise Worte sind: Sein oder Nichtsein, das ist hier die Frage?";
 
  const static char *const fragen_table[] PROGMEM = {
    fragen_0, fragen_1, fragen_2, fragen_3, fragen_4, fragen_5, fragen_6, fragen_7, fragen_8, fragen_9,
    fragen_10, fragen_11, fragen_12, fragen_13, fragen_14, fragen_15, fragen_16, fragen_17, fragen_18, fragen_19,
    fragen_20, fragen_21, fragen_22, fragen_23, fragen_24, fragen_25, fragen_26, fragen_27, fragen_28, fragen_29,
    fragen_30, fragen_31, fragen_32, fragen_33, fragen_34, fragen_35, fragen_36, fragen_37, fragen_38, fragen_39,
    fragen_40, fragen_41, fragen_42, fragen_43, fragen_44, fragen_45, fragen_46, fragen_47, fragen_48, fragen_49,
    fragen_50, fragen_51, fragen_52, fragen_53, fragen_54, fragen_55, fragen_56, fragen_57, fragen_58, fragen_59,
    fragen_60, fragen_61, fragen_62, fragen_63, fragen_64, fragen_65, fragen_66, fragen_67, fragen_68, fragen_69,
    fragen_70, fragen_71, fragen_72, fragen_73, fragen_74, fragen_75, fragen_76, fragen_77, fragen_78, fragen_79,
    fragen_80, fragen_81, fragen_82, fragen_83, fragen_84, fragen_85, fragen_86, fragen_87, fragen_88, fragen_89,
    fragen_90, fragen_91, fragen_92, fragen_93, fragen_94, fragen_95, fragen_96, fragen_97, fragen_98, fragen_99
  };

  /* Tabelle mit Antworten im flash Speicher */
  const static char antworten_0[] PROGMEM = "Windsor";
  const static char antworten_1[] PROGMEM = "-273,15";
  const static char antworten_2[] PROGMEM = "Fitzgerald";
  const static char antworten_3[] PROGMEM = "Titanen";
  const static char antworten_4[] PROGMEM = "Atlas";
  const static char antworten_5[] PROGMEM = "Mjoelnir";
  const static char antworten_6[] PROGMEM = "Reykjavik";
  const static char antworten_7[] PROGMEM = "Haus am See, Peter Fox";
  const static char antworten_8[] PROGMEM = "John Lennon, Paul McCartney";
  const static char antworten_9[] PROGMEM = "Sn";
  const static char antworten_10[] PROGMEM = "Es";
  const static char antworten_11[] PROGMEM = "N";
  const static char antworten_12[] PROGMEM = "Marone";
  const static char antworten_13[] PROGMEM = "Brille";
  const static char antworten_14[] PROGMEM = "Baguette Magique";
  const static char antworten_15[] PROGMEM = "Reisefreiheit";
  const static char antworten_16[] PROGMEM = "1263";
  const static char antworten_17[] PROGMEM = "39,5";
  const static char antworten_18[] PROGMEM = "41";
  const static char antworten_19[] PROGMEM = "Maennliche";
  const static char antworten_20[] PROGMEM = "Ottawa";
  const static char antworten_21[] PROGMEM = "Ljubljana";
  const static char antworten_22[] PROGMEM = "Konstantinopel";
  const static char antworten_23[] PROGMEM = "Nieuw Amsterdam";
  const static char antworten_24[] PROGMEM = "753 v.Chr.";
  const static char antworten_25[] PROGMEM = "Erich Kaestner";
  const static char antworten_26[] PROGMEM = "Astrid Lindgren";
  const static char antworten_27[] PROGMEM = "Efraim";
  const static char antworten_28[] PROGMEM = "Louie";
  const static char antworten_29[] PROGMEM = "Steigbuegel";
  const static char antworten_30[] PROGMEM = "6";
  const static char antworten_31[] PROGMEM = "3,14159";
  const static char antworten_32[] PROGMEM = "Anfangsbuchstaben der 3 Gruender";
  const static char antworten_33[] PROGMEM = "Hans Riedel (aus) Bonn";
  const static char antworten_34[] PROGMEM = "88";
  const static char antworten_35[] PROGMEM = "½ mv^2";
  const static char antworten_36[] PROGMEM = "Positron";
  const static char antworten_37[] PROGMEM = "0,3";
  const static char antworten_38[] PROGMEM = "Narbe";
  const static char antworten_39[] PROGMEM = "Fruchtbarkeit";
  const static char antworten_40[] PROGMEM = "Albert Einstein";
  const static char antworten_41[] PROGMEM = "Elsa Einstein";
  const static char antworten_42[] PROGMEM = "Winston Churchill ";
  const static char antworten_43[] PROGMEM = "Ludwig der XIV";
  const static char antworten_44[] PROGMEM = "Ludwig der Fromme ";
  const static char antworten_45[] PROGMEM = "Adenin, Thymin, Guanin, Cytosin ";
  const static char antworten_46[] PROGMEM = "Keins";
  const static char antworten_47[] PROGMEM = "Vatikanstadt";
  const static char antworten_48[] PROGMEM = "100 Liter ";
  const static char antworten_49[] PROGMEM = "336";
  const static char antworten_50[] PROGMEM = "30 Jahre ";
  const static char antworten_51[] PROGMEM = "Glucose ";
  const static char antworten_52[] PROGMEM = "ATP und H2O";
  const static char antworten_53[] PROGMEM = "Anfangsbuchstaben der Gruender";
  const static char antworten_54[] PROGMEM = "McGonagall";
  const static char antworten_55[] PROGMEM = "Mit dem Mund";
  const static char antworten_56[] PROGMEM = "Tom Riddle ";
  const static char antworten_57[] PROGMEM = "Muggel ";
  const static char antworten_58[] PROGMEM = "Bremen";
  const static char antworten_59[] PROGMEM = "Nutze den Tag ";
  const static char antworten_60[] PROGMEM = "Der gruene Knollenblaetterpilz";
  const static char antworten_61[] PROGMEM = "Vitamin D";
  const static char antworten_62[] PROGMEM = "Skorbut";
  const static char antworten_63[] PROGMEM = "21 ";
  const static char antworten_64[] PROGMEM = "Mohammad";
  const static char antworten_65[] PROGMEM = "Cringe";
  const static char antworten_66[] PROGMEM = "Mars";
  const static char antworten_67[] PROGMEM = "Etwa 900 Mal so lang";
  const static char antworten_68[] PROGMEM = "444";
  const static char antworten_69[] PROGMEM = "13000 Woerter";
  const static char antworten_70[] PROGMEM = "Ein Stromgenerator im Auto";
  const static char antworten_71[] PROGMEM = "10 %";
  const static char antworten_72[] PROGMEM = "Springfield";
  const static char antworten_73[] PROGMEM = "Zieh mich hoch ";
  const static char antworten_74[] PROGMEM = "Drei";
  const static char antworten_75[] PROGMEM = "486 v.Chr.";
  const static char antworten_76[] PROGMEM = "Vibranium";
  const static char antworten_77[] PROGMEM = "20";
  const static char antworten_78[] PROGMEM = "Doppelt so lang";
  const static char antworten_79[] PROGMEM = "Einhorn";
  const static char antworten_80[] PROGMEM = "Durch ihre Haut";
  const static char antworten_81[] PROGMEM = "Karotten";
  const static char antworten_82[] PROGMEM = "Millimeter Quecksilbersaeule";
  const static char antworten_83[] PROGMEM = "32";
  const static char antworten_84[] PROGMEM = "Indien";
  const static char antworten_85[] PROGMEM = "10-12%";
  const static char antworten_86[] PROGMEM = "1969";
  const static char antworten_87[] PROGMEM = "Gesellschaft mit beschraenkter Haftung ";
  const static char antworten_88[] PROGMEM = "Ural";
  const static char antworten_89[] PROGMEM = "Rotkaeppchen ";
  const static char antworten_90[] PROGMEM = "Tick, Trick und Track ";
  const static char antworten_91[] PROGMEM = "Insektenkunde";
  const static char antworten_92[] PROGMEM = "368 Meter";
  const static char antworten_93[] PROGMEM = "Die Wuerde des Menschen ist unantastbar.";
  const static char antworten_94[] PROGMEM = "a² + b² = c²";
  const static char antworten_95[] PROGMEM = "Bauspeicheldruese";
  const static char antworten_96[] PROGMEM = "Wasser";
  const static char antworten_97[] PROGMEM = "Das Ohr";
  const static char antworten_98[] PROGMEM = "Die Zunge"; 
  const static char antworten_99[] PROGMEM = "William Shakespear";

  const static char *const antworten_table[] PROGMEM = {
    antworten_0, antworten_1, antworten_2, antworten_3, antworten_4, antworten_5, antworten_6, antworten_7, antworten_8, antworten_9,
    antworten_10, antworten_11, antworten_12, antworten_13, antworten_14, antworten_15, antworten_16, antworten_17, antworten_18, antworten_19,
    antworten_20, antworten_21, antworten_22, antworten_23, antworten_24, antworten_25, antworten_26, antworten_27, antworten_28, antworten_29,
    antworten_30, antworten_31, antworten_32, antworten_33, antworten_34, antworten_35, antworten_36, antworten_37, antworten_38, antworten_39,
    antworten_40, antworten_41, antworten_42, antworten_43, antworten_44, antworten_45, antworten_46, antworten_47, antworten_48, antworten_49,
    antworten_50, antworten_51, antworten_52, antworten_53, antworten_54, antworten_55, antworten_56, antworten_57, antworten_58, antworten_59,
    antworten_60, antworten_61, antworten_62, antworten_63, antworten_64, antworten_65, antworten_66, antworten_67, antworten_68, antworten_69,
    antworten_70, antworten_71, antworten_72, antworten_73, antworten_74, antworten_75, antworten_76, antworten_77, antworten_78, antworten_79,
    antworten_80, antworten_81, antworten_82, antworten_83, antworten_84, antworten_85, antworten_86, antworten_87, antworten_88, antworten_89,
    antworten_90, antworten_91, antworten_92, antworten_93, antworten_94, antworten_95, antworten_96, antworten_97, antworten_98, antworten_99
  };

  /* Erstellen einer Zufallszahl um eine Frage zu wählen */
  randomSeed(millis());
  int randomzahl = random(100);
  
  displayAusgabe(F("Du bekommst eine Quizzfrage."));
  displayAusgabe(F("Sage deine Antwort laut und druecke dann Enter."));
  delay(500);
  displayAusgabe(F("Zum Starten Enter druecken."));

  /*Start wenn Knopfdruck (Enter Knopf)*/
  while (digitalRead(KNOPF_1) == HIGH){
    delay(10);
  }

  /* Ausgabe von Frage und Antwort */
  char buffer[100];       // Zwischenspeicher im dynamischen Speicher
  
  strcpy_P(buffer, (char *)pgm_read_word(&(fragen_table[randomzahl])));  // nötig zum dekodieren
  String frage = buffer;    // char-Array zu String
  displayAusgabe(frage);

  delay(500);
  /*Ausgabe nach Knopfdruck (Enter Knopf)*/
  while (digitalRead(KNOPF_1) == HIGH){
    delay(10);
  }
  
  strcpy_P(buffer, (char *)pgm_read_word(&(antworten_table[randomzahl]))); 
  String antwort = buffer;    // char-Array zu String
  displayAusgabe(antwort + " (Enter)");
  delay(500);
  /*Ausgabe nach Knopfdruck (Enter Knopf)*/
  while (digitalRead(KNOPF_1) == HIGH){
    delay(10);
  }

  /* Ergebnis wird eingelesen und Konsequenzen gezogen */
  displayAusgabe(F("Richtig? Gruen! Falsch? Rot!"));
  while (digitalRead(KNOPF_2) == HIGH && digitalRead(KNOPF_3) == HIGH ){
    delay(10);
  }
  if(digitalRead(KNOPF_2)==LOW){
    tonErfolg();
  }else{
    tonMisserfolg();
    schrittZurueck(spieler);
  }
}  


 
/* Aufgaben:
 * Der Spieler muss eine Aufgabe erledigen, die Mitspieler bewerten den Erfolg.
 * Bei Misserfolg muss der Spieler einen Schritt zurück gehen.
 */
void aufgabe(int spieler){
  /* Tabelle mit Aufgaben im flash Speicher */
  const static char aufgaben_0[] PROGMEM = "Singe ein Lied!";
  const static char aufgaben_1[] PROGMEM = "Beruehre mit der Zunge deine Nase!";
  const static char aufgaben_2[] PROGMEM = "Wackle mit den Ohren!";
  const static char aufgaben_3[] PROGMEM = "Sage einen Zungenbrecher auf!";
  const static char aufgaben_4[] PROGMEM = "Rufe die 10. Person in deiner Kontaktliste an und gestehe ihr deine Liebe!";
  const static char aufgaben_5[] PROGMEM = "Erzaehle von deinem peinlichsten Moment!";
  const static char aufgaben_6[] PROGMEM = "Zeige dein peinlichstes Foto!";
  const static char aufgaben_7[] PROGMEM = "Tausche bis zum Ende des Spiels ein Kleidungsstueck mit einer anderen Person!";
  const static char aufgaben_8[] PROGMEM = "Imitiere eine Person am Tisch!";
  const static char aufgaben_9[] PROGMEM = "Lasse dir von einer Person deiner Wahl ins Gesicht klatschen!";
  const static char aufgaben_10[] PROGMEM = "Teste wessen Haare am besten riechen!";
  const static char aufgaben_11[] PROGMEM = "Gaehne und versuche, eine Person am Tisch auch zum Gaehnen zu bringen!";
  const static char aufgaben_12[] PROGMEM = "Flirte mit einer Person am Tisch. Gib alles!";
  const static char aufgaben_13[] PROGMEM = "Spiele eine Szene aus deinem Lieblingsfilm!";
  const static char aufgaben_14[] PROGMEM = "Klemme dein Bein hinter deinen Kopf!";
  const static char aufgaben_15[] PROGMEM = "Trage einer Person ein Ehegeluebde vor (Augenkontakt nicht vergessen!)";
  const static char aufgaben_16[] PROGMEM = "Spiele eine Sterbeszene!";
  const static char aufgaben_17[] PROGMEM = "Mache jedem hier ein Kompliment!";
  const static char aufgaben_18[] PROGMEM = "Rappe eine Strophe über einen Mitspieler!";
  const static char aufgaben_19[] PROGMEM = "Du darfst eine Minute nicht blinzeln!";
  const static char aufgaben_20[] PROGMEM = "Mache 30 Sekunden eine Planke!";
  const static char aufgaben_21[] PROGMEM = "Singe das ABC rueckwaerts!";
  const static char aufgaben_22[] PROGMEM = "Nenne die Geburtsdaten aller Spieler!";
  const static char aufgaben_23[] PROGMEM = "Mach deine Augen zu und errate deine Mitspieler an ihren Haenden!";
  const static char aufgaben_24[] PROGMEM = "Mache mit einer Person am Tisch Schluss!";
  const static char aufgaben_25[] PROGMEM = "Imitiere deinen Lieblingscharakter!";
  const static char aufgaben_26[] PROGMEM = "Denke dir eine Geschichte ueber die Person rechts neben dir aus!";
  const static char aufgaben_27[] PROGMEM = "Bringe jeden hier zum Lachen!"; 

  const static char *const aufgaben_table[] PROGMEM = {
    aufgaben_0, aufgaben_1, aufgaben_2, aufgaben_3, aufgaben_4, aufgaben_5, aufgaben_6, aufgaben_7, aufgaben_8, aufgaben_9, 
    aufgaben_10, aufgaben_11, aufgaben_12, aufgaben_13, aufgaben_14, aufgaben_15, aufgaben_16, aufgaben_17, aufgaben_18, aufgaben_19, 
    aufgaben_20, aufgaben_21, aufgaben_22, aufgaben_23, aufgaben_24, aufgaben_25, aufgaben_26, aufgaben_27
  };

  /* Erstellen einer Zufallszahl um eine Frage zu wählen */
  randomSeed(millis());
  int randomzahl = random(28); 
  displayAusgabe(F("Du hast eine Aufgabe!"));
  delay(500);
  displayAusgabe(F("Zum Starten Enter druecken."));
     
  /* Start wenn Knopfdruck (Enter Knopf) */
  while (digitalRead(KNOPF_1) == HIGH){
    delay(10);
  }

  /* Ausgabe der Aufgabe */
  char buffer[100];       // Zwischenspeicher im dynamischen Speicher
  strcpy_P(buffer, (char *)pgm_read_word(&(aufgaben_table[randomzahl])));  // nötig zum dekodieren
  String aufgabe = buffer;    // char-Array zu String
  displayAusgabe(aufgabe + " (Enter)");
  delay(500);
  /*Ausgabe nach Knopfdruck (Enter Knopf)*/
  while (digitalRead(KNOPF_1) == HIGH){
    delay(10);
  }
  /* Ergebnis wird eingelesen und Konsequenzen gezogen */
  displayAusgabe(F("Mitspieler entscheiden: Bestanden? Gruen! Nicht bestanden? Rot!"));
  while (digitalRead(KNOPF_2) == HIGH && digitalRead(KNOPF_3) == HIGH ){
    delay(10);
  }

  if(digitalRead(KNOPF_2)==LOW){
    tonErfolg();
  }else{
    tonMisserfolg();
    schrittZurueck(spieler);
  }
}
